package com.hcl.stubAPI;

import java.net.URI;
import java.net.URISyntaxException;
import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;

import javax.net.ssl.SSLContext;

import org.apache.hc.client5.http.async.methods.SimpleHttpRequest;
import org.apache.hc.client5.http.async.methods.SimpleHttpResponse;
import org.apache.hc.client5.http.async.methods.SimpleRequestProducer;
import org.apache.hc.client5.http.async.methods.SimpleResponseConsumer;
import org.apache.hc.client5.http.impl.async.CloseableHttpAsyncClient;
import org.apache.hc.client5.http.impl.async.HttpAsyncClients;
import org.apache.hc.client5.http.impl.nio.PoolingAsyncClientConnectionManager;
import org.apache.hc.client5.http.impl.nio.PoolingAsyncClientConnectionManagerBuilder;
import org.apache.hc.client5.http.protocol.HttpClientContext;
import org.apache.hc.client5.http.ssl.ClientTlsStrategyBuilder;
import org.apache.hc.core5.concurrent.FutureCallback;
import org.apache.hc.core5.http.Method;
import org.apache.hc.core5.http.nio.ssl.TlsStrategy;
import org.apache.hc.core5.io.CloseMode;
import org.apache.hc.core5.ssl.SSLContexts;
import org.apache.hc.core5.ssl.TrustStrategy;
import org.json.JSONArray;
import org.json.JSONObject;

public class APIHelper {

	private SimpleHttpResponse getHTTPResponse(final SimpleHttpRequest request) throws InterruptedException,
			ExecutionException, KeyManagementException, NoSuchAlgorithmException, KeyStoreException {

		final PoolingAsyncClientConnectionManager cm = getClientConnectionManager();
		final CloseableHttpAsyncClient client = HttpAsyncClients.custom().setConnectionManager(cm).build();
		client.start();

		try {
			final Future<SimpleHttpResponse> future = client.execute(SimpleRequestProducer.create(request),
					SimpleResponseConsumer.create(), HttpClientContext.create(),
					new FutureCallback<SimpleHttpResponse>() {
						@Override
						public void completed(final SimpleHttpResponse response) {
							
						}

						@Override
						public void cancelled() {
							System.out.println(request + " cancelled");
						}

						@Override
						public void failed(Exception e) {
							System.out.println(request + "->" + e);
						}
					});
			return future.get();
		} finally {
			client.close(CloseMode.GRACEFUL);
		}
	}
	
	private PoolingAsyncClientConnectionManager getClientConnectionManager()
			throws NoSuchAlgorithmException, KeyManagementException, KeyStoreException {
		final SSLContext sslcontext = SSLContexts.custom().loadTrustMaterial(new TrustStrategy() {
			@Override
			public boolean isTrusted(final X509Certificate[] chain, final String authType) throws CertificateException {
				return true;
			}
		}).build();
		final TlsStrategy tlsStrategy = ClientTlsStrategyBuilder.create().setSslContext(sslcontext).build();
		final PoolingAsyncClientConnectionManager cm = PoolingAsyncClientConnectionManagerBuilder.create()
				.setTlsStrategy(tlsStrategy).build();
		return cm;
	}
	
	public void fetchAndWriteData(StubDetailsWriter csvWriter, String url) throws KeyManagementException, NoSuchAlgorithmException, KeyStoreException, InterruptedException, ExecutionException, URISyntaxException {
		SimpleHttpResponse simpleResponse = getHTTPResponse(new SimpleHttpRequest(Method.GET, new URI(url + "api/domains/")));
		if(simpleResponse.getCode() == 404) {
			System.out.println("RTCP URL is not valid, Please enter valid RTCP URL and try again!!");
			new RuntimeException();
		}
		String domains = simpleResponse.getBody().getBodyText();
		JSONArray domainArray = new JSONArray(domains);
		domainArray.forEach(domain -> {
			String domainName = ((JSONObject) domain).get("name").toString();
			JSONArray envArray = ((JSONObject) domain).getJSONArray("environments");
			envArray.forEach(env -> {
				String environment = ((JSONObject) env).get("name").toString();
				String uri = url + "api/stubs/?domainName=" + domainName + "&environment=" + environment;
				try {
					SimpleHttpResponse response = getHTTPResponse(new SimpleHttpRequest(Method.GET, new URI(uri)));
					writeStubDetails(csvWriter, domainName, response);
				} catch (Exception e1) {
					e1.printStackTrace();
				}
			});
		});
	}
	
	private static void writeStubDetails(StubDetailsWriter writer, String domainName, SimpleHttpResponse simpleResponse) {
		String body = simpleResponse.getBody().getBodyText();
		JSONArray array = new JSONArray(body);
		array.forEach(object -> {
			String[] data = { ((JSONObject) object).get("id").toString(), ((JSONObject) object).get("name").toString(),
					((JSONObject) object).get("state").toString(), ((JSONObject) object).get("version").toString(),
					domainName, ((JSONObject) object).get("environment").toString() };
			writer.write(data);
		});
		
	}

}
