/*******************************************************************************
 * Licensed Materials - Property of HCL
 * (c) Copyright HCL Technologies Ltd. 2022. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ghc.password.provider.file;

import java.io.BufferedReader;
import java.io.FileReader;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.ghc.password.provider.PasswordProvider;
import com.ghc.password.provider.PasswordProviderGUI;
import com.ghc.tags.TagDataStore;

public class FilePasswordProvider implements PasswordProvider {
	
	private static final Logger logger = Logger
	         .getLogger(FilePasswordProvider.class.getName());
	private static final String FILE_PATH = System.getProperty("java.io.tmpdir") + "FilePassword.txt" ; //Path of File

   private final TagDataStore tds;

   public FilePasswordProvider(TagDataStore tds) {
      this.tds = tds;
   }

   @Override
   public PasswordProviderGUI createGUI() {
      return new FilePasswordProviderGUI(tds);
   }

   @Override
   public String resolvePassword(String config) {
      try {
         FileReader file = new FileReader(FILE_PATH); 
         try (BufferedReader reader = new BufferedReader(file)) {
            String line = null;
            while ((line = reader.readLine()) != null) {
               if (line.contains(config)) {
                  return line.split("=")[1];
               }
            }
            logger.log(Level.SEVERE, "Invalid Credentials.");
         }
      } catch (Exception e) {
         logger.log(Level.SEVERE, "Exception Occurred While Accessing/Reading File : " + e.getMessage());
      }

      return "";
   }

}
